@@ -1,53 +1,62 @@
 from contextlib import contextmanager
 from typing import Iterator, List, Union
 
-from python_pachyderm.service import Service, transaction_proto
+import grpc
+
+from python_pachyderm.proto.v2.transaction import transaction_pb2, transaction_pb2_grpc
 
 
 def _transaction_from(transaction):
-    if isinstance(transaction, transaction_proto.Transaction):
+    if isinstance(transaction, transaction_pb2.Transaction):
         return transaction
     else:
-        return transaction_proto.Transaction(id=transaction)
+        return transaction_pb2.Transaction(id=transaction)
 
 
 class TransactionMixin:
     """A mixin for transaction-related functionality."""
 
+    _channel: grpc.Channel
+
+    def __init__(self):
+        self.__stub = transaction_pb2_grpc.APIStub(self._channel)
+        super().__init__()
+
     def batch_transaction(
-        self, requests: List[transaction_proto.TransactionRequest]
-    ) -> transaction_proto.TransactionInfo:
+        self, requests: List[transaction_pb2.TransactionRequest]
+    ) -> transaction_pb2.TransactionInfo:
         """Executes a batch transaction.
 
         Parameters
         ----------
-        requests : List[transaction_proto.TransactionRequest]
+        requests : List[transaction_pb2.TransactionRequest]
             A list of ``TransactionRequest`` protobufs. Each protobuf must
             only have one field set.
 
         Returns
         -------
-        transaction_proto.TransactionInfo
+        transaction_pb2.TransactionInfo
             A protobuf object with info on the transaction.
 
         Examples
         --------
         >>> # Deletes one repo and creates a branch in another repo atomically
         >>> client.batch_transaction([
-            transaction_proto.TransactionRequest(delete_repo=pfs_proto.DeleteRepoRequest(repo=pfs_proto.Repo(name="foo"))),
-            transaction_proto.TransactionRequest(create_branch=pfs_proto.CreateBranchRequest(branch=pfs_proto.Branch(
-                repo=pfs_proto.Repo(name="bar", type="user"), name="staging"
+            transaction_pb2.TransactionRequest(delete_repo=pfs_pb2.DeleteRepoRequest(repo=pfs_pb2.Repo(name="foo"))),
+            transaction_pb2.TransactionRequest(create_branch=pfs_pb2.CreateBranchRequest(branch=pfs_pb2.Branch(
+                repo=pfs_pb2.Repo(name="bar", type="user"), name="staging"
             )))
         ])
         """
-        return self._req(Service.TRANSACTION, "BatchTransaction", requests=requests)
+        message = transaction_pb2.BatchTransactionRequest(requests=requests)
+        return self.__stub.BatchTransaction(message)
 
-    def start_transaction(self) -> transaction_proto.Transaction:
+    def start_transaction(self) -> transaction_pb2.Transaction:
         """Starts a transaction.
 
         Returns
         -------
-        transaction_proto.Transaction
+        transaction_pb2.Transaction
             A protobuf object that represents the transaction.
 
         Examples
@@ -56,21 +65,22 @@ class TransactionMixin:
         >>> # do stuff
         >>> client.finish_transaction(transaction)
         """
-        return self._req(Service.TRANSACTION, "StartTransaction")
+        message = transaction_pb2.StartTransactionRequest()
+        return self.__stub.StartTransaction(message)
 
     def inspect_transaction(
-        self, transaction: Union[str, transaction_proto.Transaction]
-    ) -> transaction_proto.TransactionInfo:
+        self, transaction: Union[str, transaction_pb2.Transaction]
+    ) -> transaction_pb2.TransactionInfo:
         """Inspects a transaction.
 
         Parameters
         ----------
-        transaction : Union[str, transaction_proto.Transaction]
+        transaction : Union[str, transaction_pb2.Transaction]
             The ID or protobuf object representing the transaction.
 
         Returns
         -------
-        transaction_proto.TransactionInfo
+        transaction_pb2.TransactionInfo
             A protobuf object with info on the transaction.
 
         Examples
@@ -81,20 +91,19 @@ class TransactionMixin:
 
         .. # noqa: W505
         """
-        return self._req(
-            Service.TRANSACTION,
-            "InspectTransaction",
+        message = transaction_pb2.InspectTransactionRequest(
             transaction=_transaction_from(transaction),
         )
+        return self.__stub.InspectTransaction(message)
 
     def delete_transaction(
-        self, transaction: Union[str, transaction_proto.Transaction]
+        self, transaction: Union[str, transaction_pb2.Transaction]
     ) -> None:
         """Deletes a transaction.
 
         Parameters
         ----------
-        transaction : Union[str, transaction_proto.Transaction]
+        transaction : Union[str, transaction_pb2.Transaction]
             The ID or protobuf object representing the transaction.
 
         Examples
@@ -106,39 +115,40 @@ class TransactionMixin:
 
         .. # noqa: W505
         """
-        self._req(
-            Service.TRANSACTION,
-            "DeleteTransaction",
+        message = transaction_pb2.DeleteTransactionRequest(
             transaction=_transaction_from(transaction),
         )
+        self.__stub.DeleteTransaction(message)
 
     def delete_all_transactions(self) -> None:
         """Deletes all transactions."""
-        self._req(Service.TRANSACTION, "DeleteAll")
+        message = transaction_pb2.DeleteAllRequest()
+        self.__stub.DeleteAll(message)
 
-    def list_transaction(self) -> List[transaction_proto.TransactionInfo]:
+    def list_transaction(self) -> List[transaction_pb2.TransactionInfo]:
         """Lists unfinished transactions.
 
         Returns
         -------
-        List[transaction_proto.TransactionInfo]
+        List[transaction_pb2.TransactionInfo]
             A list of protobuf objects that contain info on a transaction.
         """
-        return self._req(Service.TRANSACTION, "ListTransaction").transaction_info
+        message = transaction_pb2.ListTransactionRequest()
+        return self.__stub.ListTransaction(message).transaction_info
 
     def finish_transaction(
-        self, transaction: Union[str, transaction_proto.Transaction]
-    ) -> transaction_proto.TransactionInfo:
+        self, transaction: Union[str, transaction_pb2.Transaction]
+    ) -> transaction_pb2.TransactionInfo:
         """Finishes a transaction.
 
         Parameters
         ----------
-        transaction : Union[str, transaction_proto.Transaction]
+        transaction : Union[str, transaction_pb2.Transaction]
             The ID or protobuf object representing the transaction.
 
         Returns
         -------
-        transaction_proto.TransactionInfo
+        transaction_pb2.TransactionInfo
             A protobuf object with info on the transaction.
 
         Examples
@@ -147,21 +157,20 @@ class TransactionMixin:
         >>> # do stuff
         >>> client.finish_transaction(transaction)
         """
-        return self._req(
-            Service.TRANSACTION,
-            "FinishTransaction",
-            transaction=_transaction_from(transaction),
+        message = transaction_pb2.FinishTransactionRequest(
+            transaction=_transaction_from(transaction)
         )
+        return self.__stub.FinishTransaction(message)
 
     @contextmanager
-    def transaction(self) -> Iterator[transaction_proto.Transaction]:
+    def transaction(self) -> Iterator[transaction_pb2.Transaction]:
         """A context manager for running operations within a transaction. When
         the context manager completes, the transaction will be deleted if an
         error occurred, or otherwise finished.
 
         Yields
         -------
-        transaction_proto.Transaction
+        transaction_pb2.Transaction
             A protobuf object that represents a transaction.
 
         Examples
